/* Copyright (c) 1998 by Andrew Maltsev <am@amsoft.ru>
 *
 * This is the helper for non suid programs (CGI scripts in Perl or
 * Shell for example).
 *
 * The algorithm is:
 *  1) determining what program we were called for from argv[0]
 *  2) getting the owner and the group of the program file
 *  3) changing real and effective user and group ids to the specified
 *  4) executing the program
 *
 * This is very suitable to call various CGI scripts from web server.
 * The user id of the stub must allow changing uid/gid to target ones -
 * it appears to be safe.
*/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

/* Place here nicknames and pathes to programs
*/
struct { char *nick, *path; } const nicks[]=
{ { "ruboard.pl",	"%BINDIR%/ruboard.pl" }
, { "ruboard.cgi",	"%BINDIR%/ruboard.pl" }
, { "ruboard",		"%BINDIR%/ruboard.pl" }
, { "rubadmin.pl",	"%BINDIR%/rubadmin.pl" }
, { "rubadmin.cgi",	"%BINDIR%/rubadmin.pl" }
, { "rubadmin",		"%BINDIR%/rubadmin.pl" }
, { "guestbook.pl",	"%BINDIR%/guestbook.pl" }
, { "guestbook.cgi",	"%BINDIR%/guestbook.pl" }
, { "guestbook",	"%BINDIR%/guestbook.pl" }
};

/* Main
*/
int
main(int ac, char *av[])
{ char const *ourname;
  char const *path;
  unsigned const ne=sizeof(nicks)/sizeof(*nicks);
  unsigned i;
  struct stat st;

/* Looking for path
*/
  for(ourname=av[0]+strlen(av[0])-1; ourname>av[0] && *ourname!='/'; ourname--);
  if(*ourname=='/') ourname++;
  for(i=0; i!=ne && strcmp(ourname,nicks[i].nick); i++);
  if(i==ne)
   { fprintf(stderr,"No executable found for `%s' nickname!\n"
                    "Look into source for description.\n",ourname);
     exit(1);
   }
  path=nicks[i].path;

/* Getting executable statistic
*/
  if(stat(path,&st))
   { fprintf(stderr,"%s: No such file - %s\n",ourname,path);
     exit(1);
   }

/* Changing to given uid/gid
 * (setuid/setgid are unusable in Linux, so using setreuid/setregid)
*/
  if(setregid(st.st_gid,st.st_gid) || setreuid(st.st_uid,st.st_uid))
   { fprintf(stderr,"%s: Can't suid to %lu.%lu\n"
                   ,ourname,(unsigned long)st.st_uid,(unsigned long)st.st_gid);
     exit(1);
   }
  execv(path,av);
  fprintf(stderr,"%s: Can't exec %s!\n",ourname,path);
  exit(1);
}
